/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 OW2 consortium (www.ow2.org)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLUtil.java 98 2006-02-24 16:18:48Z alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.commons.threadlocal;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;

/**
 * <p>
 * {@link Transformer} factory.
 * </p>
 * <p>
 * To avoid several instance of the same {@link Transformer} (with same options)
 * in the same Thread, each Transformer must be created using this factory.
 * </p>
 * <p>
 * When a {@link Transformer} is no longer necessary, it must be reset using
 * {@link Transformer#reset()} to be reusable.
 * </p>
 * 
 * @author Christophe DENEUX (Capgemini Sud)
 * 
 */
// TODO: To improve memory management, perhaps it could be interesting to
// replace
// ThreadLocal object by an object pool. But how to size it ?
public final class Transformers {

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The Transformer is the default one provided by the current classloader
     * (no property set).
     * </p>
     */
    private final static ThreadLocal<Transformer> defaultTransformerThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {
                return TransformerFactory.newInstance().newTransformer();
            } catch (TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The {@link Transformer} is the one provided by the JVM classloader.
     * </p>
     */
    private final static ThreadLocal<Transformer> jvmTransformerThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            final ClassLoader currentClassLoader = Thread.currentThread().getContextClassLoader();
            try {
                final ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
                Thread.currentThread().setContextClassLoader(systemClassLoader);
                return TransformerFactory.newInstance().newTransformer();

            } catch (TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            } finally {
                Thread.currentThread().setContextClassLoader(currentClassLoader);
            }
        }
    };

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The Transformer is the default one (provided by
     * {@link #defaultTransformerThreadLocal}).
     * </p>
     * <p>
     * When the {@link Transformer} is no longuer necessary, it must be reset
     * using {@link Transformer#reset()} to be reuseable.
     * </p>
     */
    public final static Transformer getDefaultTransformer() {
        return Transformers.defaultTransformerThreadLocal.get();
    }

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The Transformer is the one provided by
     * {@link #defaultTransformerThreadLocal}.
     * </p>
     * <p>
     * When the {@link Transformer} is no longuer necessary, it must be reset
     * using {@link Transformer#reset()} to be reuseable.
     * </p>
     */
    public final static Transformer getXmlWithDeclarationTransformer() {
        Transformer t = Transformers.defaultTransformerThreadLocal.get();
        t.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
        t.setOutputProperty(OutputKeys.METHOD, "xml");
        return t;
    }

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The Transformer is the one provided by
     * {@link #defaultTransformerThreadLocal}.
     * </p>
     * <p>
     * When the {@link Transformer} is no longuer necessary, it must be reset
     * using {@link Transformer#reset()} to be reuseable.
     * </p>
     */
    public final static Transformer getXmlWithoutDeclarationTransformer() {
        Transformer t = Transformers.defaultTransformerThreadLocal.get();
        t.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        t.setOutputProperty(OutputKeys.METHOD, "xml");
        return t;
    }

    /**
     * <p>
     * {@link Transformer} as thread local.
     * </p>
     * <p>
     * The Transformer is the one provided by
     * {@link #defaultTransformerThreadLocal}.
     * </p>
     * <p>
     * When the {@link Transformer} is no longuer necessary, it must be reset
     * using {@link Transformer#reset()} to be reuseable.
     * </p>
     */
    public final static Transformer getWithoutDeclarationTransformer() {
        Transformer t = Transformers.defaultTransformerThreadLocal.get();
        t.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        return t;
    }
}
